/*
 * Trasmetti_peso.ino  Legge il peso sulla bilancia e lo trasmette periodicamente; 
 * legge solo gli incrementi
 * Hardware: Feather 32u4 LoRa; HX711;  Bilancia con cella di carico a 4 fili portata 7 Kg
 * 
 * Pacchetto trasmesso
radiopacket[0]  =  highByte(ID_TX);
radiopacket[1]  =  lowByte(ID_TX);
radiopacket[2]  =  highByte(Len);
radiopacket[3]  =  lowByte(Len);
radiopacket[4]  =  highByte(vbatTX);
radiopacket[5]  =  lowByte(vbatTX);
radiopacket[6]  =  highByte(npacket);
radiopacket[7]  =  lowByte(npacket);
radiopacket[8] =   highByte(pesog);
radiopacket[9] =   lowByte(pesog);

 * Paolo Bonelli  25/03/2018
 */
#include <SPI.h>
#include <RH_RF95.h>
#include "HX711.h"

//  parametri bilancia
#define DT_pin 12
#define SCK_pin 11
HX711 scale(DT_pin,SCK_pin);

/* for feather32u4 */
#define RFM95_CS 8
#define RFM95_RST 4
#define RFM95_INT 7
#define VBATPIN A9       //  Pin where you can measure the battery power 
#define packetLength 10  // lunghezza in byte del pacchetto trasmesso
// Singleton instance of the radio driver
RH_RF95 rf95(RFM95_CS, RFM95_INT);

////////////////////////////////  MONITOR /////////////////////////////////////
byte ECHO = 0;          // mettere a 1 solo quando è collegato a cavo USB 
////////////////////////////      BILANCIA   /////////////////////////////////
//  fattore di calibrazione iniziale, poi viene cambiato in base al totale del peso
// dalla funzione CalcolaFattore

float fattore_ini =  247.68;    //  bilancia cinese B1
word ID_TX =         48001;     //Messina B1

//float fattore_ini =  240.70;    //  bilancia cinese B2
//word ID_TX =         48002;     //Messina B2

//float fattore_ini =  238.33;    //  bilancia cinese B3
//word ID_TX =         48000;     //Messina B3

//float fattore_ini =    241.82;    //  bilancia cinese B4
//word ID_TX =           48004;     //Messina B4

////float fattore_ini = 388.37;  // LoRa_lowCost Pluviometro
float fattore = fattore_ini;    
////////////////////////////////   RADIO   ////////////////////////////////////
///////////////////////////////////////////////////////////////////////
#define RF95_FREQ 868.0  // frequenza della portante
////////////////////////////////////////////////////////////////////////
#define Tpower 23        // potenza della trasmissione 0...23
////////////////////////////////////////////////////////////////////////////////
byte  radiopacket[packetLength]; // pacchetto trasmissione
word vbatTX = 0;
word npacket=0;
///////////////////////////////   VARIABILI BILANCIA ////////////////////////////
float pesog = 0;               //  valore in grammi della pesata
int i = 0;
///////////////////////////////////////////////////////////////////
void setup() {
  // settaggio pin
  pinMode(RFM95_RST, OUTPUT);  // equivale al tasto del reset
  digitalWrite(RFM95_RST, HIGH);

// interfaccia seriale per monitor
  if(ECHO){
    while (!Serial);    
    Serial.begin(9600);
    delay(100);
    Serial.println("Trasmetti_peso_1.ino"); 
  }
  
 //  reset radio
  digitalWrite(RFM95_RST, LOW);
  delay(10);
  digitalWrite(RFM95_RST, HIGH);
  delay(10);
  
// inizializzazione radio
  while (!rf95.init()) {
    
  if(ECHO){
    Serial.println("LoRa radio init failed");
  }
  }
  if(ECHO){
  Serial.println("LoRa radio init OK!");
  }
// Defaults after init are 434.0MHz, 13dBm, Bw = 125 kHz, Cr = 4/5, Sf = 128chips/symbol, CRC on
// The default transmitter power is 13dBm, using PA_BOOST.
// If you are using RFM95/96/97/98 modules which uses the PA_BOOST transmitter pin, then 
// you can set transmitter powers from 5 to 23 dBm:
// Set frequency. Defaults after init are 434.0MHz, modulation GFSK_Rb250Fd250, +13dbM
  rf95.setFrequency(RF95_FREQ);
  
  if(ECHO){
  Serial.print("Set Freq to: "); Serial.println(RF95_FREQ);
  } 
  
//  Set transmission Power
  rf95.setTxPower(Tpower, false);
  
  if(ECHO){
  Serial.print("Set Power to: "); Serial.println(Tpower);
  } 

// inizializzazione bilancia
  scale.set_gain(128);
  delay(1000);
  scale.tare();
  scale.set_scale(fattore);   
  scale.tare();
}
/////////////////////////////////////////////////////////////////
void loop() {

  //  pesa e azzera la bilancia
     CampionaPeso();
     if(ECHO){
             Serial.println();
             Serial.print(" pesog = ");Serial.println(pesog);
     }
     Trasmetti();
    delay(20000);
}
