/*
 * Trasmetti_vibrazioni.ino  
 * Si sveglia con l'interrupt alla prima vibrazione e conta il numero di vibrazioni 
 * in un periodo di tempo prefissato.
 * Hardware: Feather 32u4 LoRa; Sensore 801S
 * 
 *    Wiring:
   feather             S801
   3.3                  Vcc
   GND                  GND
   2 (Interrupt)       DO  (Digital Output)

 * Pacchetto trasmesso
radiopacket[0]  =  highByte(ID_TX);
radiopacket[1]  =  lowByte(ID_TX);
radiopacket[2]  =  highByte(Len);
radiopacket[3]  =  lowByte(Len);
radiopacket[4]  =  highByte(vbatTX);
radiopacket[5]  =  lowByte(vbatTX);
radiopacket[6]  =  highByte(npacket);
radiopacket[7]  =  lowByte(npacket);
radiopacket[8] =   highByte(n);        //numero di impulsi provenienti dal sensore in un certo tempo prefissato sPeriod in secondi
radiopacket[9] =   lowByte(n);
radiopacket[10] =  highByte(sPeriod);  // intervallo di tempo in secondi entro cui si contano gli impulsi
radiopacket[11] =  lowByte(sPeriod);

 * Paolo Bonelli  28/03/2018
 */
#include <SPI.h>
#include <RH_RF95.h>
#include "Adafruit_SleepyDog.h"

/* for feather32u4 */
#define RFM95_CS 8
#define RFM95_RST 4
#define RFM95_INT 7
#define VBATPIN A9       //  Pin where you can measure the battery power 
#define RF95_FREQ 868.0  // frequenza della portante
#define Tpower 23        // potenza della trasmissione 0...23
// Singleton instance of the radio driver
RH_RF95 rf95(RFM95_CS, RFM95_INT);

////////////////////////////////  MONITOR /////////////////////////////////////
byte ECHO = 0;          // mettere a 1 solo quando è collegato a cavo USB 
////////////////////////////////   RADIO   ////////////////////////////////////
//word ID_TX = 49001;   //   Vibrazione V01                    byte 12; Trasmetti_vibrazioni.ino
//word ID_TX = 49002;   //   Vibrazione V02                    byte 12; 
//word ID_TX = 49003;   //   Vibrazione V03                    byte 12; 
word ID_TX = 49004;   //   Vibrazione V04                    byte 12; 
#define packetLength 12            // lunghezza in byte del pacchetto trasmesso
byte    radiopacket[packetLength]; // pacchetto trasmissione
word    vbatTX = 0;
word    npacket = 0;
///////////////////////////////   VARIABILI BILANCIA ////////////////////////////
int   i =           0;
word  Len =         0;
volatile word n =   0;
int   interruptPin =  2;
float TempoOld =    0;
float Tempo =       0;
float Period =      10000;  // periodo in cui vengono contatti gli impulsi
word  ncicli = 0;
///////////////////////////////////////////////////////////////////
void setup() {
  // settaggio pin
  pinMode(RFM95_RST, OUTPUT);  // equivale al tasto del reset
  digitalWrite(RFM95_RST, HIGH);
  pinMode(interruptPin, INPUT);
  digitalWrite(interruptPin, LOW);  // forse non serve
  
// interfaccia seriale per monitor
  if(ECHO){
    while (!Serial);    
    Serial.begin(57600);
    delay(100);
    Serial.println("Trasmetti_vibrazioni.ino"); 
  }
  
 //  reset radio
  digitalWrite(RFM95_RST, LOW);
  delay(10);
  digitalWrite(RFM95_RST, HIGH);
  delay(10);
  
// inizializzazione radio
  while (!rf95.init()) {
    
  if(ECHO){
    Serial.println("LoRa radio init failed");
  }
  }
  if(ECHO){
  Serial.println("LoRa radio init OK!");
  }
// Defaults after init are 434.0MHz, 13dBm, Bw = 125 kHz, Cr = 4/5, Sf = 128chips/symbol, CRC on
// The default transmitter power is 13dBm, using PA_BOOST.
// If you are using RFM95/96/97/98 modules which uses the PA_BOOST transmitter pin, then 
// you can set transmitter powers from 5 to 23 dBm:
// Set frequency. Defaults after init are 434.0MHz, modulation GFSK_Rb250Fd250, +13dbM
  
  rf95.setFrequency(RF95_FREQ);
  
  if(ECHO){
  Serial.print("Set Freq to: "); Serial.println(RF95_FREQ);
  } 
 if(ECHO){
  Serial.print("Set Power to: "); Serial.println(Tpower);
  } 

//  Set transmission Power
  rf95.setTxPower(Tpower, false);
  
  attachInterrupt(digitalPinToInterrupt(interruptPin), count, RISING);
}
/////////////////////////////////////////////////////////////////
void loop() {
  ncicli++;
  Tempo = 0;
  // loop che dura Period (millisecondi) durante i quali conta gli impulsi
  while (Tempo < Period) {
    Tempo = millis() - TempoOld;
  }
  TempoOld = millis();
  detachInterrupt(digitalPinToInterrupt(interruptPin));
  
  // Se non ha contato niente e ha fatto almeno 3 giri, trasmette un messaggio con n=0;
 if( ncicli > 3 && n == 0){
    Trasmetti();
    ncicli = 0;
    n = 0;
  }
  
  // Se  ha contato qualcosa, trasmette un messaggio con il valore di n;
  if(n > 0){
    Trasmetti();
    n = 0;
    ncicli = 0;
  }
  attachInterrupt(digitalPinToInterrupt(interruptPin), count, RISING);

  //  Mette a dormire CPU e modulo radio per circe 30 secondi
 if(ECHO){
  delay(10000);
 }else{
  rf95.sleep();
  Watchdog.sleep(8000);  // 0.31 mA di consumo. 8000 ms è il max, ma posso ripeterlo più volte
  Watchdog.sleep(8000);  
  Watchdog.sleep(8000);  
  Watchdog.sleep(6000);  
 }
}
